clear all   
cap log close
set more off, perm
set scheme s2manual

* Set current directory
cd /disk/agedisk3/medicare.work/finkels-DUA22559/kluender/

* Set date
global date 071015
* Set location of input data
global inputs mergefiles/finaldata
* Set location for output. Create a subfolder here named "output_${date}$.
global outputs Documentation
* This is the file extension for the figure. 
* The figures in the paper are generated in excel from the output of this program.
global file_extension eps

* Turn "on" sections of the code
scalar data_prep = 1
scalar analysis = 1

* Set the buffer region around the kink
local kink = 200
local kink2 = `kink'-400


* DATA PREPARATION AND COLLAPSE
if data_prep==1 {

* Pull in baseline data
use ${inputs}/bene070809_baseline, clear

* Drop if individuals are missing risk scores (will happen if they were not present in prior year)
drop if missing(RXHCC1)

* Create number of HCC turned "on" by ICD9 diagnoses. See CMS documentation for more information on HCCs.
egen rxhcc_total = rowtotal(RXHCC*)
	replace rxhcc_total = 5 if rxhcc_total>5
forv tot = 0/5 {
	gen total_rxhccs_`tot' = rxhcc_total==`tot'
}

* Merge in the coefficients from the risk score software using dummy variable
gen mergevar = 1
merge m:1 mergevar using Programs/CalculatingRiskScoresAndUtilization/RxHCC_2012software/rxcoeff_tomerge

* Summary Statistics
sum totcost, det
sum oopspend, det
count if totcost==0
count if oopspend==0
tab female, mi
sum age

* Keep only individuals within $2000 of the kink for our exercises
gen sample = icl_diff_widebin>=-2000 & icl_diff_widebin<2000
tab sample
keep if sample

* Create the number of hospital visits in 6 months of the next year (2008-2010)
foreach var in op er_op ip er_ip { 
	egen `var'_visits_tplus1 = rowtotal(`var'_admits_mo1-`var'_admits_mo6)
}

* Generate variable for dying in next year
gen death_yr = year(death_dt)
gen die_nextyear = (death_yr == yr+1)
egen all_visits_tplus1 = rowtotal(*_visits_tplus1)
egen er_visits_tplus1 = rowtotal(er_*_visits_tplus1)
drop er_op* er_ip*
gen freq = 1
gen black = race==2
gen asian = race==4
gen hisp = race==5

* Generate bins used by risk score programs with Medicare data
gen F65_69 = age>=65 & age<=69 & female
gen F70_74 = age>=70 & age<=74 & female
gen F75_79 = age>=75 & age<=79 & female
gen F80_84 = age>=80 & age<=84 & female
gen F85_89 = age>=85 & age<=89 & female
gen F90_94 = age>=90 & age<=94 & female
gen F95_GT = age>=95 & female
gen M65_69 = age>=65 & age<=69 & !female
gen M70_74 = age>=70 & age<=74 & !female
gen M75_79 = age>=75 & age<=79 & !female
gen M80_84 = age>=80 & age<=84 & !female
gen M85_89 = age>=85 & age<=89 & !female
gen M90_94 = age>=90 & age<=94 & !female
gen M95_GT = age>=95 & !female

* This set of the code assigns the coefficient from the CMS risk score program for the relevant age/gender bin.
gen AGE_GENDER_CE = .
foreach var of varlist M* F* {
	tab `var' Rx_CE_NoLowAged_`var', mi
	replace AGE_GENDER_CE = `var'*Rx_CE_NoLowAged_`var' if `var'
}
tab AGE_GENDER_CE, mi
sum AGE_GENDER_CE
local mean_AG_CE = r(mean)
gen AG_CE_demeaned = AGE_GENDER_CE-`mean_AG_CE'

tab icl_diff_widebin, mi

* Create copies of variables to get standard errors in the collapse
foreach var of varlist female age white *_visits_tplus1 riskscore* die* total_rxhccs_* RXHCC* Rx_CE_* M* F* AG_CE_demeaned{
	gen se_`var' = `var'
}

* Collapse data to the bin level
collapse (mean) totcost female age white all_visits_tplus1 op_visits_tplus1 ip_visits_tplus1 er_visits_tplus1 riskscore* ///
	die* total_rxhccs_* RXHCC* Rx_CE_* M* F* AG_CE_demeaned ///
	(sem) se_female se_age se_white se_*_visits_tplus1 se_riskscore* se_die* se_total_rxhccs_* se_RXHCC* se_Rx_CE_* se_M* se_F* se_AG_CE_demeaned ///
	(sum) freq, by(icl_diff_widebin)

* Regress the indicidence levels leading up to the kink to predict incidence through the kink
foreach var of varlist female age white all_visits_tplus1 op_visits_tplus1 ip_visits_tplus1 er_visits_tplus1 riskscore* die* total_rxhccs_* RXHCC* M* F* AG_CE_demeaned{
	reg `var' icl_diff_widebin if icl_diff_widebin<-`kink' [fw=freq] 
		predict p`var'
		predict se_p`var', stdp	
}

* Generate confidence intervals for the figures
foreach var of varlist female age white all_visits_tplus1 op_visits_tplus1 ip_visits_tplus1 er_visits_tplus1 riskscore* die* total_rxhccs_* RXHCC* M* F* AG_CE_demeaned {
	gen CI_lb_`var' = `var'-1.96*se_`var'
	gen CI_ub_`var' = `var'+1.96*se_`var'
	gen CI_lb_p`var' = p`var'-1.96*se_p`var'
	gen CI_ub_p`var' = p`var'+1.96*se_p`var'
}

drop se_*

* Outsheet the data for the figures
outsheet * using "mergefiles/RiskScoreOutputs/multidimensional_kink`kink'.csv", c replace

save "mergefiles/RiskScoreOutputs/multidimensional_kink`kink'.dta", replace

} // end if data_prep==1




if analysis == 1 {

use mergefiles/RiskScoreOutputs/multidimensional_kink`kink', clear

* Move this to data prep at some point
foreach var of varlist M* F* {
	replace Rx_CE_NoLowAged_`var' = AG_CE_demeaned if `var'
}

global positives ""
global negatives ""

* CONSTRUCTED RISK SCORES
local varlist pos_dft = ""
local varlist neg_dft
foreach var of varlist F* M* RXHCC* {
	sum `var' if icl_diff_widebin>=-`kink' & icl_diff_widebin<-`kink2'
	local actual = r(mean)
	sum p`var' if icl_diff_widebin>=-`kink' & icl_diff_widebin<-`kink2'
	local predicted = r(mean)
	
	* This section determines whether incidence is higher or lower than predicted through the kink
	local dev = (`actual'-`predicted')*Rx_CE_NoLowAged_`var'

	if `dev'>0 {
		global positives "${positives} `var'"
	}
	if `dev'<0 {
		global negatives "${negatives} `var'"
	}
}

di "${positives}"
di "${negatives}"

gen constructed_rs_pos = `mean_AG_CE'
gen constructed_rs_neg = `mean_AG_CE'
gen constructed_rs = `mean_AG_CE'
gen constructed_rs_fit = `mean_AG_CE'

* Collect the positive deviations to generate a risk score holding the negatives at their predicted values
foreach coef of global positives {
	replace constructed_rs_pos = constructed_rs_pos + `coef'*Rx_CE_NoLowAged_`coef'
	replace constructed_rs_neg = constructed_rs_neg + p`coef'*Rx_CE_NoLowAged_`coef'
	replace constructed_rs = constructed_rs + `coef'*Rx_CE_NoLowAged_`coef'
	replace constructed_rs_fit = constructed_rs_fit + p`coef'*Rx_CE_NoLowAged_`coef'
}

* Do the same for negative deviations
foreach coef of global negatives {
	replace constructed_rs_pos = constructed_rs_pos + p`coef'*Rx_CE_NoLowAged_`coef'
	replace constructed_rs_neg = constructed_rs_neg + `coef'*Rx_CE_NoLowAged_`coef'
	replace constructed_rs = constructed_rs + `coef'*Rx_CE_NoLowAged_`coef'
	replace constructed_rs_fit = constructed_rs_fit + p`coef'*Rx_CE_NoLowAged_`coef'
}

* Plot just as a sanity check. The figures are put together from the outsheeted output in excel
twoway (scatter constructed_rs_pos icl_diff_widebin if icl_diff_widebin<-`kink2', msize(medsmall)) (scatter constructed_rs_neg icl_diff_widebin if icl_diff_widebin<-`kink2', msize(medsmall)) (scatter constructed_rs icl_diff_widebin if icl_diff_widebin<-`kink2', xline(-`kink',lpattern(dash)) ///
	legend(label(1 "Includes Positive Deviations") label(2 "Include Negative Deviations") label(3 "Constructed Risk Score (Includes Both)") rows(3)) xtitle("ICL Difference"))
graph export "Documentation/output_${date}/Constructed_RiskScores_kink`kink'.${file_extension}", replace

* Outsheet the results
outsheet constructed_rs_pos constructed_rs constructed_rs_neg icl_diff_widebin using "Documentation/output_${date}/Constructed_RiskScores_kink`kink'.txt", replace
outsheet * using "Documentation/output_${date}/Multidimensional_Graphs_kink`kink'.txt", replace

* INDIVIDUAL HCC TABLE AND GRAPHS
* This section puts together Table 1
local outcomes = "die* all_visits_tplus1 op_visits_tplus1 ip_visits_tplus er_visits_tplus1"
local demos = "age white"
local risk = "total_rxhccs_* RXHCC* riskscore*"

keep if icl_diff_widebin>=-`kink' & icl_diff_widebin<-`kink2'

sum constructed_rs_pos
local tot_pos = r(mean)
sum constructed_rs_neg 
local tot_neg = r(mean)
sum constructed_rs_fit
local tot = r(mean)

local avg_positives = `tot_pos'-`tot'
local avg_negatives = `tot_neg'-`tot'

foreach deviation in positives negatives {

local i = 0
foreach var of global `deviation' {
	local i = `i'+1
}
matrix define DFT_`deviation' = J(`i', 5, .)
matrix colnames DFT_`deviation' = "KinkIncidence" "TrendIncidence" "DFT" "RS_DFT" "Pct RS_DFT"
local i = 0
local rownames ""
foreach var of global `deviation' {
	local i = `i'+1
	local rownames "`rownames' `var'"
	sum `var'
	local actual = r(mean)
	sum p`var'
	local predicted = r(mean)
	local dev = `actual'-`predicted'
	sum Rx_CE_NoLowAged_`var'
	local coef = r(mean)
	local rs_dev = `dev'*`coef'
	local rs_dev_pct = 100*(`rs_dev'/`avg_`deviation'')
	
	matrix DFT_`deviation'[`i', 1] = `actual'
	matrix DFT_`deviation'[`i', 2] = `predicted'
	matrix DFT_`deviation'[`i', 3] = `dev'
	matrix DFT_`deviation'[`i', 4] = `rs_dev'
	matrix DFT_`deviation'[`i', 5] = `rs_dev_pct'
	
}
matrix rownames DFT_`deviation' = `rownames'
matrix list DFT_`deviation'

preserve 
clear
svmat2 DFT_`deviation', rnames(variable) names(col)
outsheet * using "Documentation/output_${date}/DFT_`deviation'_table_kink`kink'.txt", replace
restore

} // end foreach deviation

} // analysis == 1

